@@ -28,6 +28,7 @@
 #include <sycl/ext/oneapi/functional.hpp>
 #if defined(__NVPTX__)
 #include <sycl/ext/oneapi/experimental/cuda/non_uniform_algorithms.hpp>
+#include <sycl/ext/oneapi/experimental/non_uniform_groups.hpp>
 #endif
 #endif
 
@@ -59,42 +60,25 @@ template <> inline id<3> linear_id_to_id(range<3> r, size_t linear_id) {
 }
 
 // ---- get_local_linear_range
-template <typename Group> size_t get_local_linear_range(Group g);
-template <> inline size_t get_local_linear_range<group<1>>(group<1> g) {
-  return g.get_local_range(0);
-}
-template <> inline size_t get_local_linear_range<group<2>>(group<2> g) {
-  return g.get_local_range(0) * g.get_local_range(1);
-}
-template <> inline size_t get_local_linear_range<group<3>>(group<3> g) {
-  return g.get_local_range(0) * g.get_local_range(1) * g.get_local_range(2);
-}
-template <>
-inline size_t get_local_linear_range<sycl::sub_group>(sycl::sub_group g) {
-  return g.get_local_range()[0];
+template <typename Group> inline auto get_local_linear_range(Group g) {
+  auto local_range = g.get_local_range();
+  auto result = local_range[0];
+  for (size_t i = 1; i < Group::dimensions; ++i)
+    result *= local_range[i];
+  return result;
 }
 
 // ---- get_local_linear_id
-template <typename Group>
-inline typename Group::linear_id_type get_local_linear_id(Group g);
-
+template <typename Group> inline auto get_local_linear_id(Group g) {
 #ifdef __SYCL_DEVICE_ONLY__
-#define __SYCL_GROUP_GET_LOCAL_LINEAR_ID(D)                                    \
-  template <>                                                                  \
-  inline group<D>::linear_id_type get_local_linear_id<group<D>>(group<D>) {    \
-    nd_item<D> it = sycl::detail::Builder::getNDItem<D>();                     \
-    return it.get_local_linear_id();                                           \
+  if constexpr (std::is_same_v<Group, group<1>> ||
+                std::is_same_v<Group, group<2>> ||
+                std::is_same_v<Group, group<3>>) {
+    auto it = sycl::detail::Builder::getNDItem<Group::dimensions>();
+    return it.get_local_linear_id();
   }
-__SYCL_GROUP_GET_LOCAL_LINEAR_ID(1);
-__SYCL_GROUP_GET_LOCAL_LINEAR_ID(2);
-__SYCL_GROUP_GET_LOCAL_LINEAR_ID(3);
-#undef __SYCL_GROUP_GET_LOCAL_LINEAR_ID
 #endif // __SYCL_DEVICE_ONLY__
-
-template <>
-inline sycl::sub_group::linear_id_type
-get_local_linear_id<sycl::sub_group>(sycl::sub_group g) {
-  return g.get_local_id()[0];
+  return g.get_local_linear_id();
 }
 
 // ---- is_native_op
@@ -116,14 +100,19 @@ template <typename T, typename BinaryOperation> struct is_native_op {
 // ---- is_plus
 template <typename T, typename BinaryOperation>
 using is_plus = std::integral_constant<
-    bool, std::is_same_v<BinaryOperation, sycl::plus<T>> ||
-              std::is_same_v<BinaryOperation, sycl::plus<void>>>;
+    bool,
+    std::is_same_v<BinaryOperation, sycl::plus<std::remove_const_t<T>>> ||
+        std::is_same_v<BinaryOperation, sycl::plus<std::add_const_t<T>>> ||
+        std::is_same_v<BinaryOperation, sycl::plus<void>>>;
 
 // ---- is_multiplies
 template <typename T, typename BinaryOperation>
 using is_multiplies = std::integral_constant<
-    bool, std::is_same_v<BinaryOperation, sycl::multiplies<T>> ||
-              std::is_same_v<BinaryOperation, sycl::multiplies<void>>>;
+    bool,
+    std::is_same_v<BinaryOperation, sycl::multiplies<std::remove_const_t<T>>> ||
+        std::is_same_v<BinaryOperation,
+                       sycl::multiplies<std::add_const_t<T>>> ||
+        std::is_same_v<BinaryOperation, sycl::multiplies<void>>>;
 
 // ---- is_complex
 // Use SFINAE so that the "true" branch could be implemented in
@@ -134,9 +123,9 @@ struct is_complex : public std::false_type {};
 
 // ---- is_arithmetic_or_complex
 template <typename T>
-using is_arithmetic_or_complex = std::integral_constant<
-    bool, sycl::detail::is_complex<typename std::remove_cv_t<T>>::value ||
-              sycl::detail::is_arithmetic<T>::value>;
+using is_arithmetic_or_complex =
+    std::integral_constant<bool, sycl::detail::is_complex<T>::value ||
+                                     sycl::detail::is_arithmetic<T>::value>;
 
 template <typename T>
 struct is_vector_arithmetic_or_complex
@@ -534,14 +523,17 @@ joint_none_of(Group g, Ptr first, Ptr last, Predicate pred) {
 // TODO: remove check for detail::is_vec<T> once sycl::vec is trivially
 // copyable.
 template <typename Group, typename T>
-std::enable_if_t<(std::is_same_v<std::decay_t<Group>, sub_group> &&
+std::enable_if_t<((std::is_same_v<std::decay_t<Group>, sub_group> ||
+                   sycl::ext::oneapi::experimental::is_user_constructed_group_v<
+                       std::decay_t<Group>>) &&
                   (std::is_trivially_copyable_v<T> ||
                    detail::is_vec<T>::value)),
                  T>
-shift_group_left(Group, T x, typename Group::linear_id_type delta = 1) {
+shift_group_left(Group g, T x, typename Group::linear_id_type delta = 1) {
 #ifdef __SYCL_DEVICE_ONLY__
-  return sycl::detail::spirv::SubgroupShuffleDown(x, delta);
+  return sycl::detail::spirv::ShuffleDown(g, x, delta);
 #else
+  (void)g;
   (void)x;
   (void)delta;
   throw sycl::exception(make_error_code(errc::feature_not_supported),
@@ -553,14 +545,17 @@ shift_group_left(Group, T x, typename Group::linear_id_type delta = 1) {
 // TODO: remove check for detail::is_vec<T> once sycl::vec is trivially
 // copyable.
 template <typename Group, typename T>
-std::enable_if_t<(std::is_same_v<std::decay_t<Group>, sub_group> &&
+std::enable_if_t<((std::is_same_v<std::decay_t<Group>, sub_group> ||
+                   sycl::ext::oneapi::experimental::is_user_constructed_group_v<
+                       std::decay_t<Group>>) &&
                   (std::is_trivially_copyable_v<T> ||
                    detail::is_vec<T>::value)),
                  T>
-shift_group_right(Group, T x, typename Group::linear_id_type delta = 1) {
+shift_group_right(Group g, T x, typename Group::linear_id_type delta = 1) {
 #ifdef __SYCL_DEVICE_ONLY__
-  return sycl::detail::spirv::SubgroupShuffleUp(x, delta);
+  return sycl::detail::spirv::ShuffleUp(g, x, delta);
 #else
+  (void)g;
   (void)x;
   (void)delta;
   throw sycl::exception(make_error_code(errc::feature_not_supported),
@@ -572,14 +567,17 @@ shift_group_right(Group, T x, typename Group::linear_id_type delta = 1) {
 // TODO: remove check for detail::is_vec<T> once sycl::vec is trivially
 // copyable.
 template <typename Group, typename T>
-std::enable_if_t<(std::is_same_v<std::decay_t<Group>, sub_group> &&
+std::enable_if_t<((std::is_same_v<std::decay_t<Group>, sub_group> ||
+                   sycl::ext::oneapi::experimental::is_user_constructed_group_v<
+                       std::decay_t<Group>>) &&
                   (std::is_trivially_copyable_v<T> ||
                    detail::is_vec<T>::value)),
                  T>
-permute_group_by_xor(Group, T x, typename Group::linear_id_type mask) {
+permute_group_by_xor(Group g, T x, typename Group::linear_id_type mask) {
 #ifdef __SYCL_DEVICE_ONLY__
-  return sycl::detail::spirv::SubgroupShuffleXor(x, mask);
+  return sycl::detail::spirv::ShuffleXor(g, x, mask);
 #else
+  (void)g;
   (void)x;
   (void)mask;
   throw sycl::exception(make_error_code(errc::feature_not_supported),
@@ -591,14 +589,17 @@ permute_group_by_xor(Group, T x, typename Group::linear_id_type mask) {
 // TODO: remove check for detail::is_vec<T> once sycl::vec is trivially
 // copyable.
 template <typename Group, typename T>
-std::enable_if_t<(std::is_same_v<std::decay_t<Group>, sub_group> &&
+std::enable_if_t<((std::is_same_v<std::decay_t<Group>, sub_group> ||
+                   sycl::ext::oneapi::experimental::is_user_constructed_group_v<
+                       std::decay_t<Group>>) &&
                   (std::is_trivially_copyable_v<T> ||
                    detail::is_vec<T>::value)),
                  T>
-select_from_group(Group, T x, typename Group::id_type local_id) {
+select_from_group(Group g, T x, typename Group::id_type local_id) {
 #ifdef __SYCL_DEVICE_ONLY__
-  return sycl::detail::spirv::SubgroupShuffle(x, local_id);
+  return sycl::detail::spirv::Shuffle(g, x, local_id);
 #else
+  (void)g;
   (void)x;
   (void)local_id;
   throw sycl::exception(make_error_code(errc::feature_not_supported),
@@ -766,25 +767,6 @@ exclusive_scan_over_group(Group g, T x, BinaryOperation binary_op) {
 
 // four argument version of exclusive_scan_over_group is specialized twice
 // once for vector_arithmetic, once for (scalar_arithmetic || complex)
-template <typename Group, typename V, typename T, class BinaryOperation>
-std::enable_if_t<(is_group_v<std::decay_t<Group>> &&
-                  detail::is_vector_arithmetic_or_complex<V>::value &&
-                  detail::is_vector_arithmetic_or_complex<T>::value &&
-                  detail::is_native_op<V, BinaryOperation>::value &&
-                  detail::is_native_op<T, BinaryOperation>::value),
-                 T>
-exclusive_scan_over_group(Group g, V x, T init, BinaryOperation binary_op) {
-  static_assert(std::is_same_v<decltype(binary_op(init, x)), T>,
-                "Result type of binary_op must match scan accumulation type.");
-  T result;
-  typename detail::get_scalar_binary_op<BinaryOperation>::type
-      scalar_binary_op{};
-  for (int s = 0; s < x.size(); ++s) {
-    result[s] = exclusive_scan_over_group(g, x[s], init[s], scalar_binary_op);
-  }
-  return result;
-}
-
 template <typename Group, typename V, typename T, class BinaryOperation>
 std::enable_if_t<
     (is_group_v<std::decay_t<Group>> &&
@@ -816,6 +798,25 @@ exclusive_scan_over_group(Group g, V x, T init, BinaryOperation binary_op) {
 #endif
 }
 
+template <typename Group, typename V, typename T, class BinaryOperation>
+std::enable_if_t<(is_group_v<std::decay_t<Group>> &&
+                  detail::is_vector_arithmetic_or_complex<V>::value &&
+                  detail::is_vector_arithmetic_or_complex<T>::value &&
+                  detail::is_native_op<V, BinaryOperation>::value &&
+                  detail::is_native_op<T, BinaryOperation>::value),
+                 T>
+exclusive_scan_over_group(Group g, V x, T init, BinaryOperation binary_op) {
+  static_assert(std::is_same_v<decltype(binary_op(init, x)), T>,
+                "Result type of binary_op must match scan accumulation type.");
+  T result;
+  typename detail::get_scalar_binary_op<BinaryOperation>::type
+      scalar_binary_op{};
+  for (int s = 0; s < x.size(); ++s) {
+    result[s] = exclusive_scan_over_group(g, x[s], init[s], scalar_binary_op);
+  }
+  return result;
+}
+
 // ---- joint_exclusive_scan
 template <typename Group, typename InPtr, typename OutPtr, typename T,
           class BinaryOperation>
@@ -896,23 +897,6 @@ joint_exclusive_scan(Group g, InPtr first, InPtr last, OutPtr result,
 //   this function has two overloads, one with three arguments and one with four
 //   the three argument version is specialized thrice: vector, scalar, and
 //   complex
-template <typename Group, typename T, class BinaryOperation>
-std::enable_if_t<(is_group_v<std::decay_t<Group>> &&
-                  detail::is_vector_arithmetic_or_complex<T>::value &&
-                  detail::is_native_op<T, BinaryOperation>::value),
-                 T>
-inclusive_scan_over_group(Group g, T x, BinaryOperation binary_op) {
-  static_assert(std::is_same_v<decltype(binary_op(x, x)), T>,
-                "Result type of binary_op must match scan accumulation type.");
-  T result;
-  typename detail::get_scalar_binary_op<BinaryOperation>::type
-      scalar_binary_op{};
-  for (int s = 0; s < x.size(); ++s) {
-    result[s] = inclusive_scan_over_group(g, x[s], scalar_binary_op);
-  }
-  return result;
-}
-
 template <typename Group, typename T, class BinaryOperation>
 std::enable_if_t<(is_group_v<std::decay_t<Group>> &&
                   (detail::is_scalar_arithmetic<T>::value ||
@@ -940,6 +924,23 @@ inclusive_scan_over_group(Group g, T x, BinaryOperation binary_op) {
 #endif
 }
 
+template <typename Group, typename T, class BinaryOperation>
+std::enable_if_t<(is_group_v<std::decay_t<Group>> &&
+                  detail::is_vector_arithmetic_or_complex<T>::value &&
+                  detail::is_native_op<T, BinaryOperation>::value),
+                 T>
+inclusive_scan_over_group(Group g, T x, BinaryOperation binary_op) {
+  static_assert(std::is_same_v<decltype(binary_op(x, x)), T>,
+                "Result type of binary_op must match scan accumulation type.");
+  T result;
+  typename detail::get_scalar_binary_op<BinaryOperation>::type
+      scalar_binary_op{};
+  for (int s = 0; s < x.size(); ++s) {
+    result[s] = inclusive_scan_over_group(g, x[s], scalar_binary_op);
+  }
+  return result;
+}
+
 // complex specializaiton
 template <typename Group, typename T, class BinaryOperation>
 std::enable_if_t<(is_group_v<std::decay_t<Group>> &&
